@@ -1,6 +1,14 @@
+import warnings
+
+from scrapy.exceptions import ScrapyDeprecationWarning
 from scrapy.utils.http import decode_chunked_transfer
 
 
+warnings.warn("Module `scrapy.downloadermiddlewares.chunked` is deprecated, "
+              "chunked transfers are supported by default.",
+              ScrapyDeprecationWarning, stacklevel=2)
+
+
 class ChunkedTransferMiddleware(object):
     """This middleware adds support for chunked transfer encoding, as
     documented in: http://en.wikipedia.org/wiki/Chunked_transfer_encoding
