@@ -5,13 +5,14 @@ discovering (through HTTP headers) to base Response class.
 See documentation in docs/topics/request-response.rst
 """
 
+import six
 from six.moves.urllib.parse import urljoin
 
 from w3lib.encoding import html_to_unicode, resolve_encoding, \
     html_body_declared_encoding, http_content_type_encoding
 from scrapy.http.response import Response
 from scrapy.utils.response import get_base_url
-from scrapy.utils.python import memoizemethod_noargs
+from scrapy.utils.python import memoizemethod_noargs, to_native_str
 
 
 class TextResponse(Response):
@@ -26,18 +27,18 @@ class TextResponse(Response):
         super(TextResponse, self).__init__(*args, **kwargs)
 
     def _set_url(self, url):
-        if isinstance(url, unicode):
-            if self.encoding is None:
-                raise TypeError('Cannot convert unicode url - %s has no encoding' %
-                    type(self).__name__)
-            self._url = url.encode(self.encoding)
+        if isinstance(url, six.text_type):
+            if six.PY2 and self.encoding is None:
+                raise TypeError("Cannot convert unicode url - %s "
+                                "has no encoding" % type(self).__name__)
+            self._url = to_native_str(url, self.encoding)
         else:
             super(TextResponse, self)._set_url(url)
 
     def _set_body(self, body):
-        self._body = ''
-        if isinstance(body, unicode):
-            if self.encoding is None:
+        self._body = b''  # used by encoding detection
+        if isinstance(body, six.text_type):
+            if self._encoding is None:
                 raise TypeError('Cannot convert unicode body - %s has no encoding' %
                     type(self).__name__)
             self._body = body.encode(self._encoding)
@@ -58,7 +59,12 @@ class TextResponse(Response):
 
     def body_as_unicode(self):
         """Return body as unicode"""
-        # check for self.encoding before _cached_ubody just in
+        return self.text
+
+    @property
+    def text(self):
+        """ Body as unicode """
+        # access self.encoding before _cached_ubody to make sure
         # _body_inferred_encoding is called
         benc = self.encoding
         if self._cached_ubody is None:
@@ -73,14 +79,14 @@ class TextResponse(Response):
 
     @memoizemethod_noargs
     def _headers_encoding(self):
-        content_type = self.headers.get('Content-Type')
-        return http_content_type_encoding(content_type)
+        content_type = self.headers.get(b'Content-Type', b'')
+        return http_content_type_encoding(to_native_str(content_type))
 
     def _body_inferred_encoding(self):
         if self._cached_benc is None:
-            content_type = self.headers.get('Content-Type')
-            benc, ubody = html_to_unicode(content_type, self.body, \
-                    auto_detect_fun=self._auto_detect_fun, \
+            content_type = to_native_str(self.headers.get(b'Content-Type', b''))
+            benc, ubody = html_to_unicode(content_type, self.body,
+                    auto_detect_fun=self._auto_detect_fun,
                     default_encoding=self._DEFAULT_ENCODING)
             self._cached_benc = benc
             self._cached_ubody = ubody
@@ -105,8 +111,8 @@ class TextResponse(Response):
             self._cached_selector = Selector(self)
         return self._cached_selector
 
-    def xpath(self, query):
-        return self.selector.xpath(query)
+    def xpath(self, query, **kwargs):
+        return self.selector.xpath(query, **kwargs)
 
     def css(self, query):
         return self.selector.css(query)
