@@ -6,12 +6,15 @@ This package contains a collection of Link Extractors.
 For more info see docs/topics/link-extractors.rst
 """
 import re
-from six.moves.urllib.parse import urlparse
 
-from scrapy.selector.csstranslator import ScrapyHTMLTranslator
-from scrapy.utils.url import url_is_from_any_domain
-from scrapy.utils.url import canonicalize_url, url_is_from_any_domain, url_has_any_extension
+from six.moves.urllib.parse import urlparse
+from parsel.csstranslator import HTMLTranslator
+from w3lib.url import canonicalize_url
+
 from scrapy.utils.misc import arg_to_iter
+from scrapy.utils.url import (
+    url_is_from_any_domain, url_has_any_extension,
+)
 
 
 # common file extensions that are not followed if they occur in links
@@ -28,7 +31,8 @@ IGNORED_EXTENSIONS = [
     'm4a',
 
     # office suites
-    'xls', 'xlsx', 'ppt', 'pptx', 'doc', 'docx', 'odt', 'ods', 'odg', 'odp',
+    'xls', 'xlsx', 'ppt', 'pptx', 'pps', 'doc', 'docx', 'odt', 'ods', 'odg',
+    'odp',
 
     # other
     'css', 'pdf', 'exe', 'bin', 'rss', 'zip', 'rar',
@@ -36,21 +40,23 @@ IGNORED_EXTENSIONS = [
 
 
 _re_type = type(re.compile("", 0))
-_matches = lambda url, regexs: any((r.search(url) for r in regexs))
-_is_valid_url = lambda url: url.split('://', 1)[0] in set(['http', 'https', 'file'])
+_matches = lambda url, regexs: any(r.search(url) for r in regexs)
+_is_valid_url = lambda url: url.split('://', 1)[0] in {'http', 'https', 'file'}
 
 
 class FilteringLinkExtractor(object):
 
-    _csstranslator = ScrapyHTMLTranslator()
+    _csstranslator = HTMLTranslator()
 
     def __init__(self, link_extractor, allow, deny, allow_domains, deny_domains,
                  restrict_xpaths, canonicalize, deny_extensions, restrict_css):
 
         self.link_extractor = link_extractor
 
-        self.allow_res = [x if isinstance(x, _re_type) else re.compile(x) for x in arg_to_iter(allow)]
-        self.deny_res = [x if isinstance(x, _re_type) else re.compile(x) for x in arg_to_iter(deny)]
+        self.allow_res = [x if isinstance(x, _re_type) else re.compile(x)
+                          for x in arg_to_iter(allow)]
+        self.deny_res = [x if isinstance(x, _re_type) else re.compile(x)
+                         for x in arg_to_iter(deny)]
 
         self.allow_domains = set(arg_to_iter(allow_domains))
         self.deny_domains = set(arg_to_iter(deny_domains))
@@ -62,7 +68,7 @@ class FilteringLinkExtractor(object):
         self.canonicalize = canonicalize
         if deny_extensions is None:
             deny_extensions = IGNORED_EXTENSIONS
-        self.deny_extensions = set(['.' + e for e in arg_to_iter(deny_extensions)])
+        self.deny_extensions = {'.' + e for e in arg_to_iter(deny_extensions)}
 
     def _link_allowed(self, link):
         if not _is_valid_url(link.url):
@@ -87,8 +93,8 @@ class FilteringLinkExtractor(object):
         if self.deny_domains and url_is_from_any_domain(url, self.deny_domains):
             return False
 
-        allowed = [regex.search(url) for regex in self.allow_res] if self.allow_res else [True]
-        denied = [regex.search(url) for regex in self.deny_res] if self.deny_res else []
+        allowed = (regex.search(url) for regex in self.allow_res) if self.allow_res else [True]
+        denied = (regex.search(url) for regex in self.deny_res) if self.deny_res else []
         return any(allowed) and not any(denied)
 
     def _process_links(self, links):
@@ -102,5 +108,6 @@ class FilteringLinkExtractor(object):
     def _extract_links(self, *args, **kwargs):
         return self.link_extractor._extract_links(*args, **kwargs)
 
+
 # Top-level imports
 from .lxmlhtml import LxmlLinkExtractor as LinkExtractor
